@@ -1,9 +1,7 @@
 #ifndef BIGMEMORY_UTIL_HPP
 #define BIGMEMORY_UTIL_HPP
 
-#include <vector>
-#include <string>
-#include <Rdefines.h>
+#include <Rcpp.h>
 
 #include "bigmemoryDefines.h"
 
@@ -20,6 +18,8 @@ std::string RChar2String(SEXP str);
 
 SEXP StringVec2RChar( const vector<string> &strVec );
 
+// Removed because no longer required with Rcpp
+/*
 template<typename T>
 SEXP StringVec2RChar( const vector<string> &strVec,
   T indices, const index_type indicesLength )
@@ -36,18 +36,33 @@ SEXP StringVec2RChar( const vector<string> &strVec,
   UNPROTECT(1);
   return ret;
 }
+*/
 
+#undef length
+#include <Rcpp.h>
 
 template<typename T>
 struct NewVec;
 
 template<>
 struct NewVec<int>
-{SEXP operator()(index_type n) const {return NEW_INTEGER(n);};};
+{SEXP operator()(index_type n) const {return Rcpp::IntegerVector(n);};};
+
+
+inline
+SEXP NEW_FLOAT(index_type n)
+{
+    std::vector<float> out (n, 0);
+    return Rcpp::wrap(out);
+}
+
+template<>
+struct NewVec<float>
+{SEXP operator()(index_type n) const {return NEW_FLOAT(n);};};
 
 template<>
 struct NewVec<double>
-{SEXP operator()(index_type n) const {return NEW_NUMERIC(n);};};
+{SEXP operator()(index_type n) const {return Rcpp::NumericVector(n);};};
 
 template<typename T>
 struct VecPtr;
@@ -56,8 +71,13 @@ template<>
 struct VecPtr<int>
 {int* operator()(SEXP vec) const {return INTEGER_DATA(vec);};};
 
+template<>
+struct VecPtr<float>
+{float* operator()(SEXP vec) const {return (float *)(vec);};};
+
 template<>
 struct VecPtr<double>
 {double* operator()(SEXP vec) const {return NUMERIC_DATA(vec);};};
 
+#define length(x) Rf_length(x)
 #endif // BIGMEMORY_UTIL_HPP
